@@ -29,12 +29,9 @@ use postgres::{Connection, TlsMode};
 use sqlparser::dialect::PostgreSqlDialect;
 use sqlparser::tokenizer::{Token, Tokenizer, Word};
 
+use postgres::types::Type;
 use rdbc;
-
-/// Convert a Postgres error into an RDBC error
-fn to_rdbc_err(e: &postgres::error::Error) -> rdbc::Error {
-    rdbc::Error::General(format!("{:?}", e))
-}
+use rdbc::Column;
 
 pub struct PostgresDriver {}
 
@@ -63,6 +60,10 @@ impl PConnection {
 }
 
 impl rdbc::Connection for PConnection {
+    fn create(&mut self, sql: &str) -> rdbc::Result<Rc<RefCell<dyn rdbc::Statement + '_>>> {
+        self.prepare(sql)
+    }
+
     fn prepare(&mut self, sql: &str) -> rdbc::Result<Rc<RefCell<dyn rdbc::Statement + '_>>> {
         // translate SQL, mapping ? into $1 style bound param placeholder
         let dialect = PostgreSqlDialect {};
@@ -112,26 +113,37 @@ impl<'a> rdbc::Statement for PStatement<'a> {
             .query(&self.sql, params.as_slice())
             .map_err(|e| to_rdbc_err(&e))
             .map(|rows| {
-                Rc::new(RefCell::new(PResultSet { i: 0, rows })) as Rc<RefCell<dyn rdbc::ResultSet>>
+                let meta = rows
+                    .columns()
+                    .iter()
+                    .map(|c| rdbc::Column::new(c.name(), to_rdbc_type(c.type_())))
+                    .collect();
+
+                Rc::new(RefCell::new(PResultSet { meta, i: 0, rows }))
+                    as Rc<RefCell<dyn rdbc::ResultSet>>
             })
     }
 
-    fn execute_update(&mut self, params: &Vec<rdbc::Value>) -> rdbc::Result<usize> {
+    fn execute_update(&mut self, params: &Vec<rdbc::Value>) -> rdbc::Result<u64> {
         let params = to_postgres_value(params);
         let params: Vec<&dyn postgres::types::ToSql> = params.iter().map(|v| v.as_ref()).collect();
         self.conn
             .execute(&self.sql, params.as_slice())
             .map_err(|e| to_rdbc_err(&e))
-            .map(|n| n as usize)
     }
 }
 
 struct PResultSet {
+    meta: Vec<Column>,
     i: usize,
     rows: Rows,
 }
 
 impl rdbc::ResultSet for PResultSet {
+    fn meta_data(&self) -> rdbc::Result<Rc<dyn rdbc::ResultSetMetaData>> {
+        Ok(Rc::new(self.meta.clone()))
+    }
+
     fn next(&mut self) -> bool {
         if self.i < self.rows.len() {
             self.i = self.i + 1;
@@ -141,12 +153,25 @@ impl rdbc::ResultSet for PResultSet {
         }
     }
 
-    fn get_i32(&self, i: usize) -> Option<i32> {
-        self.rows.get(self.i - 1).get(i - 1)
+    fn get_i32(&self, i: u64) -> Option<i32> {
+        self.rows.get(self.i - 1).get(i as usize - 1)
     }
 
-    fn get_string(&self, i: usize) -> Option<String> {
-        self.rows.get(self.i - 1).get(i - 1)
+    fn get_string(&self, i: u64) -> Option<String> {
+        self.rows.get(self.i - 1).get(i as usize - 1)
+    }
+}
+
+/// Convert a Postgres error into an RDBC error
+fn to_rdbc_err(e: &postgres::error::Error) -> rdbc::Error {
+    rdbc::Error::General(format!("{:?}", e))
+}
+
+fn to_rdbc_type(ty: &Type) -> rdbc::DataType {
+    match ty.name() {
+        "" => rdbc::DataType::Bool,
+        //TODO all types
+        _ => rdbc::DataType::Varchar,
     }
 }
 
@@ -157,6 +182,7 @@ fn to_postgres_value(values: &Vec<rdbc::Value>) -> Vec<Box<dyn postgres::types::
             rdbc::Value::String(s) => Box::new(s.clone()) as Box<dyn postgres::types::ToSql>,
             rdbc::Value::Int32(n) => Box::new(*n) as Box<dyn postgres::types::ToSql>,
             rdbc::Value::UInt32(n) => Box::new(*n) as Box<dyn postgres::types::ToSql>,
+            //TODO all types
         })
         .collect()
 }
@@ -191,7 +217,7 @@ mod tests {
         Ok(())
     }
 
-    fn execute(sql: &str, values: &Vec<rdbc::Value>) -> rdbc::Result<usize> {
+    fn execute(sql: &str, values: &Vec<rdbc::Value>) -> rdbc::Result<u64> {
         println!("Executing '{}' with {} params", sql, values.len());
         let driver = PostgresDriver::new();
         let conn = driver.connect("postgres://rdbc:secret@127.0.0.1:5433")?;
